#!/usr/bin/python2

import os
import commands
import gettext
import string

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk, GdkPixbuf

from mintupload_core import *

# i18n
gettext.install("mintupload", "/usr/share/linuxmint/locale")

# Location of the ui file
UI_FILE = "/usr/share/linuxmint/mintupload/manager_window.ui"


class ManagerWindow:

    def __init__(self):
        self.builder = Gtk.Builder()
        self.builder.add_from_file(UI_FILE)

        self.manager_window = self.builder.get_object("manager_window")
        self.manager_window.set_title(_("Upload Manager"))
        self.manager_window.set_icon_name(ICON)
        self.manager_window.connect("delete_event", Gtk.main_quit)

        treeview_services = self.builder.get_object("treeview_services")

        # the treeview
        column1 = Gtk.TreeViewColumn(_("Upload services"), Gtk.CellRendererText(), text=0)
        column1.set_sort_column_id(0)
        column1.set_resizable(True)

        treeview_services.append_column(column1)
        treeview_services.set_headers_clickable(True)
        treeview_services.set_reorderable(False)
        treeview_services.show()

        self.reload_services(treeview_services)

        self.builder.get_object("button_close").connect("clicked", Gtk.main_quit)
        self.builder.get_object("toolbutton_add").connect("clicked", self.add_service, treeview_services)
        self.builder.get_object("toolbutton_edit").connect("clicked", self.edit_service_from_button, treeview_services)
        self.builder.get_object("toolbutton_remove").connect("clicked", self.remove_service, treeview_services)

        treeview_services.connect("row_activated", self.edit_service_from_tree, treeview_services)

        fileMenu = Gtk.MenuItem.new_with_mnemonic(_("_File"))
        fileSubmenu = Gtk.Menu()
        fileMenu.set_submenu(fileSubmenu)
        closeMenuItem = Gtk.ImageMenuItem(Gtk.STOCK_CLOSE)
        closeMenuItem.set_use_stock(True)
        closeMenuItem.set_label(_("Close"))
        closeMenuItem.connect("activate", Gtk.main_quit)
        fileSubmenu.append(closeMenuItem)

        helpMenu = Gtk.MenuItem.new_with_mnemonic(_("_Help"))
        helpSubmenu = Gtk.Menu()
        helpMenu.set_submenu(helpSubmenu)
        aboutMenuItem = Gtk.ImageMenuItem(Gtk.STOCK_ABOUT)
        aboutMenuItem.set_use_stock(True)
        aboutMenuItem.set_label(_("About"))
        aboutMenuItem.connect("activate", self.open_about)
        helpSubmenu.append(aboutMenuItem)

        self.builder.get_object("menubar1").append(fileMenu)
        self.builder.get_object("menubar1").append(helpMenu)
        self.builder.get_object("manager_window").show_all()

    def open_about(self, widget):
        dlg = Gtk.AboutDialog()
        dlg.set_transient_for(self.manager_window)
        dlg.set_title(_("About") + " - mintupload")
        version = commands.getoutput("/usr/lib/linuxmint/common/version.py mintupload")
        dlg.set_version(version)
        dlg.set_program_name("mintupload")
        dlg.set_comments(_("Upload Manager"))

        try:
            h = open('/usr/share/common-licenses/GPL', 'r')
            s = h.readlines()
            gpl = ""
            for line in s:
                gpl += line
            h.close()
            dlg.set_license(gpl)
        except Exception, detail:
            print detail

        dlg.set_authors([
                        "Clement Lefebvre <root@linuxmint.com>",
                        "Philip Morrell <mintupload.emorrp1@mamber.net>",
                        "Manuel Sandoval <manuel@slashvar.com>",
                        "Dennis Schwertel <s@digitalkultur.net>"
                        ])
        dlg.set_icon_name(ICON)
        dlg.set_logo_icon_name(ICON)

        def close(w, res):
            if res == Gtk.ResponseType.DELETE_EVENT:
                w.destroy()

        dlg.connect("response", close)
        dlg.show()

    def response_to_dialog(self, entry, dialog, response):
        dialog.response(response)

    def add_service(self, widget, treeview_services):
        dialog = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT, Gtk.MessageType.QUESTION, Gtk.ButtonsType.OK_CANCEL, None)
        dialog.set_transient_for(self.manager_window)
        dialog.set_title(_("Upload Manager"))
        dialog.set_icon_name(ICON)
        dialog.set_border_width(6)
        dialog.set_markup(_("<b>Please enter a name for the new upload service:</b>"))
        entry = Gtk.Entry()
        entry.connect("changed", self.check_service_name, dialog)
        hbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
        label = Gtk.Label(_("Service name:"))
        hbox.pack_start(label, False, 5, 5)
        hbox.pack_end(entry, True, True, 0)
        dialog.format_secondary_markup(_("<i>Try to avoid spaces and special characters...</i>"))
        dialog.vbox.pack_end(hbox, True, True, 0)
        dialog.show_all()
        response = dialog.run()

        if response == Gtk.ResponseType.OK:
            sname = entry.get_text()

        dialog.destroy()

        if response == Gtk.ResponseType.OK:
            service = Service('/usr/share/linuxmint/mintupload/sample.service')
            if os.path.exists(config_paths['user'] + sname):
                sname += " 2"
                while os.path.exists(config_paths['user'] + sname):
                    next = int(sname[-1:]) + 1
                    sname = sname[:-1] + str(next)
            service.filename = config_paths['user'] + sname
            service.write()
            self.services.append(service)
            model = treeview_services.get_model()
            iter = model.insert_before(None, None)
            model.set_value(iter, 0, sname)
            self.edit_service(treeview_services, model.get_path(iter))

    def check_service_name(self, entry, dialog):
        text = entry.get_text()
        valid = True
        if text == "":
            valid = False
        if " " in text:
            valid = False
        invalidChars = set(string.punctuation.replace("_", ""))
        if any(char in invalidChars for char in text):
            valid = False
        dialog.get_widget_for_response(Gtk.ResponseType.OK).set_sensitive(valid)

    def remove_service(self, widget, treeview_services):
        (model, iter) = treeview_services.get_selection().get_selected()
        self.services = read_services()

        if iter != None:
            service = model.get_value(iter, 0)
            for s in self.services:
                if s['name'] == service:
                    s.remove()
                    self.services.remove(s)
            model.remove(iter)

    def reload_services(self, treeview_services):
        model = Gtk.TreeStore(str)
        model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
        treeview_services.set_model(model)

        self.services = read_services()
        for service in self.services:
            iter = model.insert_before(None, None)
            model.set_value(iter, 0, service['name'])
        del model

    def edit_service_from_tree(self, widget, path, column, treeview_services):
        self.edit_service(treeview_services, path)

    def edit_service_from_button(self, widget, treeview_services):
        selection = treeview_services.get_selection()
        (model, iter) = selection.get_selected()
        if iter is not None:
            self.edit_service(treeview_services, model.get_path(iter))

    def edit_service(self, treeview_services, path):
        model = treeview_services.get_model()
        iter = model.get_iter(path)
        sname = model.get_value(iter, 0)
        file = config_paths['user'] + sname

        dialog_edit_service = self.builder.get_object("dialog_edit_service")
        dialog_edit_service.set_transient_for(self.manager_window)
        dialog_edit_service.set_title(_("%s Properties") % sname)
        dialog_edit_service.set_icon_name(ICON)

        self.builder.get_object("button_verify").set_label(_("Check connection"))
        self.builder.get_object("button_verify").connect("clicked", self.check_connection, file)
        self.builder.get_object("button_cancel").connect("clicked", self.close_window, self.builder.get_object("dialog_edit_service"))

        #i18n
        self.builder.get_object("lbl_type").set_label(_("Type:"))
        self.builder.get_object("lbl_hostname").set_label(_("Host:"))
        self.builder.get_object("lbl_port").set_label(_("Port:"))
        self.builder.get_object("lbl_username").set_label(_("User:"))
        self.builder.get_object("lbl_password").set_label(_("Password:"))
        self.builder.get_object("lbl_timestamp").set_label(_("Timestamp:"))
        self.builder.get_object("lbl_path").set_label(_("Path:"))

        self.builder.get_object("lbl_hostname").set_tooltip_text(_("Hostname or IP address, default: ") + defaults['host'])
        self.builder.get_object("txt_host").set_tooltip_text(_("Hostname or IP address, default: ") + defaults['host'])
        self.builder.get_object("txt_host").connect("focus-out-event", self.change, file)

        self.builder.get_object("lbl_port").set_tooltip_text(_("Remote port, default is 21 for FTP, 22 for SFTP and SCP"))
        self.builder.get_object("txt_port").set_tooltip_text(_("Remote port, default is 21 for FTP, 22 for SFTP and SCP"))
        self.builder.get_object("txt_port").connect("focus-out-event", self.change, file)

        self.builder.get_object("lbl_username").set_tooltip_text(_("Username, defaults to your local username"))
        self.builder.get_object("txt_user").set_tooltip_text(_("Username, defaults to your local username"))
        self.builder.get_object("txt_user").connect("focus-out-event", self.change, file)

        self.builder.get_object("lbl_password").set_tooltip_text(_("Password, by default: password-less SCP connection, null-string FTP connection, ~/.ssh keys used for SFTP connections"))
        self.builder.get_object("txt_pass").set_tooltip_text(_("Password, by default: password-less SCP connection, null-string FTP connection, ~/.ssh keys used for SFTP connections"))
        self.builder.get_object("txt_pass").connect("focus-out-event", self.change, file)

        self.builder.get_object("lbl_timestamp").set_tooltip_text(_("Timestamp format (strftime). By default:") + defaults['format'])
        self.builder.get_object("txt_format").set_tooltip_text(_("Timestamp format (strftime). By default:") + defaults['format'])
        self.builder.get_object("txt_format").connect("focus-out-event", self.change, file)

        self.builder.get_object("lbl_path").set_tooltip_text(_("Directory to upload to. <TIMESTAMP> is replaced with the current timestamp, following the timestamp format given. By default: ."))
        self.builder.get_object("txt_path").set_tooltip_text(_("Directory to upload to. <TIMESTAMP> is replaced with the current timestamp, following the timestamp format given. By default: ."))
        self.builder.get_object("txt_path").connect("focus-out-event", self.change, file)

        try:
            config = Service(file)
            try:
                model = self.builder.get_object("combo_type").get_model()
                iter = model.get_iter_first()

                while (iter != None and model.get_value(iter, 0).lower() != config['type'].lower()):
                    iter = model.iter_next(iter)

                self.builder.get_object("combo_type").set_active_iter(iter)
                self.builder.get_object("combo_type").connect("changed", self.change, None, file)
            except:
                pass
            try:
                self.builder.get_object("txt_host").set_text(config['host'])
            except:
                self.builder.get_object("txt_host").set_text("")
            try:
                self.builder.get_object("txt_port").set_text(str(config['port']))
            except:
                self.builder.get_object("txt_port").set_text("")
            try:
                self.builder.get_object("txt_user").set_text(config['user'])
            except:
                self.builder.get_object("txt_user").set_text("")
            try:
                self.builder.get_object("txt_pass").set_text(config['pass'])
            except:
                self.builder.get_object("txt_pass").set_text("")
            try:
                self.builder.get_object("txt_format").set_text(config['format'])
            except:
                self.builder.get_object("txt_format").set_text("")
            try:
                self.builder.get_object("txt_path").set_text(config['path'])
            except:
                self.builder.get_object("txt_path").set_text("")
        except Exception, detail:
            print detail

        dialog_edit_service.run()
        dialog_edit_service.hide()

    def check_connection(self, widget, file):
        service = Service(file)
        os.system("mintupload \"" + service['name'] + "\" /usr/share/linuxmint/mintupload/mintupload.readme &")

    def get_port_for_service(self, type):
        num = "21" if type in ("Mint", "FTP") else "22"

        self.builder.get_object("txt_port").set_text(num)
        return num

    def change(self, widget, event, file):
        try:
            wname = Gtk.Buildable.get_name(widget)

            if wname == "combo_type":
                model = widget.get_model()
                iter = widget.get_active_iter()
                config = {'type': model.get_value(iter, 0).lower(),
                          'port': self.get_port_for_service(model.get_value(iter, 0))}
            else:
                config = {wname[4:]: widget.get_text()}

            s = Service(file)
            s.merge(config)
            s.write()
        except Exception as e:
            try:
                raise CustomError(_("Could not save configuration change"), e)
            except:
                pass

    def close_window(self, widget, window):
        window.hide()

window = ManagerWindow()
Gtk.main()



#!/usr/bin/python2

import os
import sys
import gettext
import time
import threading
import urllib

import gi
gi.require_version("Gtk", "3.0")
gi.require_version("Notify", "0.7")
gi.require_version("AppIndicator3", "0.1")
from gi.repository import Gtk, Gdk, GLib
from gi.repository import AppIndicator3 as AppIndicator
from mintupload_core import *

# i18n
gettext.install("mintupload", "/usr/share/linuxmint/locale")

# Name of the icon used by the indicator
SYSTRAY_ICON = "mintupload-tray"

class MainClass:

    def __init__(self):
        self.drop_zones = {}
        self.status_icon = AppIndicator.Indicator.new("mintupload", _("Upload services"), AppIndicator.IndicatorCategory.APPLICATION_STATUS)
        self.status_icon.set_status(AppIndicator.IndicatorStatus.ACTIVE)
        self.status_icon.set_icon(SYSTRAY_ICON)
        self.status_icon.set_title(_("Upload services"))

        try:
            if os.environ["XDG_CURRENT_DESKTOP"].lower() == "mate":
                self.status_icon.set_icon("up")
        except Exception, detail:
            print detail

        self.services = None

        self.build_services_menu()
        # Refresh list of services in the menu every 2 seconds
        GLib.timeout_add_seconds(2, self.reload_services)

    def reload_services(self):
        has_changed = read_services() != self.services

        if has_changed and not self.menu.is_visible():
            self.build_services_menu()

        return True

    def build_services_menu(self):
        self.services = read_services()
        self.menu = Gtk.Menu()
        services_menuitem = Gtk.MenuItem()
        title = Gtk.Label()
        title.set_text("<b><span foreground=\"grey\">" + _("Services:") + "</span></b>")
        title.set_justify(Gtk.Justification.LEFT)
        title.set_alignment(0, 0.5)
        title.set_use_markup(True)
        services_menuitem.add(title)
        self.menu.append(services_menuitem)

        for service in self.services:
            service_menuitem = Gtk.MenuItem(label="   " + service['name'])
            service_menuitem.connect("activate", self.create_drop_zone, service)
            self.menu.append(service_menuitem)

        self.menu.append(Gtk.SeparatorMenuItem())

        upload_manager_menuitem = Gtk.MenuItem(_("Upload manager..."))
        upload_manager_menuitem.connect('activate', self.launch_manager)
        self.menu.append(upload_manager_menuitem)

        self.menu.append(Gtk.SeparatorMenuItem())

        menu_item = Gtk.MenuItem()
        menu_item = Gtk.ImageMenuItem(Gtk.STOCK_CLOSE)
        menu_item.set_use_stock(True)
        menu_item.set_label(_("Quit"))
        menu_item.connect('activate', self.quit_cb)
        self.menu.append(menu_item)
        self.menu.show_all()
        self.status_icon.set_menu(self.menu)

    def launch_manager(self, widget):
        os.system("/usr/lib/linuxmint/mintupload/upload-manager.py &")

    def create_drop_zone(self, widget, service):
        if service['name'] not in self.drop_zones.keys():
            drop_zone = DropZone(service, self.drop_zones)
            self.drop_zones[service['name']] = drop_zone
        else:
            self.drop_zones[service['name']].show()

    def quit_cb(self, widget):
        Gtk.main_quit()
        sys.exit(0)

class DropZone:

    def __init__(self, service, drop_zones):
        self.service = service
        self.drop_zones = drop_zones
        self.w = Gtk.Window()

        TARGET_TYPE_TEXT = 80

        self.w.drag_dest_set(
            Gtk.DestDefaults.ALL,  # Motion, Highlight, Drop
            [Gtk.TargetEntry.new("text/uri-list", 0, TARGET_TYPE_TEXT)],
            Gdk.DragAction.MOVE | Gdk.DragAction.COPY
        )

        self.w.connect('drag-motion', self.motion_cb)
        self.w.connect('drag-drop', self.drop_cb)
        self.w.connect('drag-data-received', self.drop_data_received_cb)
        self.w.connect('destroy', self.destroy_cb)

        self.w.set_icon_name(SYSTRAY_ICON)
        self.w.set_title(self.service['name'])
        self.w.set_keep_above(True)
        self.w.set_type_hint(Gdk.WindowTypeHint.UTILITY)
        self.w.set_skip_pager_hint(True)
        self.w.set_skip_taskbar_hint(True)
        self.w.stick()

        display = Gdk.Display.get_default()
        (screen, x, y, mods) = display.get_pointer()
        self.w.move(x-50, y-50)

        self.label = Gtk.Label()
        self.label.set_text("<small>" + _("Drag &amp; Drop here to upload to %s") % self.service['name'] + "</small>")
        self.label.set_line_wrap(True)
        self.label.set_use_markup(True)
        self.label.set_width_chars(20)
        self.w.add(self.label)

        self.w.set_default_size(100, 50)

        if self.w.is_composited():
            self.w.set_opacity(0.5)

        self.w.show_all()

    def show(self):
        self.w.show_all()
        self.w.present()

    def motion_cb(self, wid, context, x, y, time):
        Gdk.drag_status(context, Gdk.DragAction.COPY, time)
        return True

    def drop_cb(self, wid, context, x, y, time):
        context.finish(True, False, time)
        return True

    def drop_data_received_cb(self, widget, context, x, y, selection, targetType, time):
        data = selection.get_data()

        filenames = []
        files = data.split('\n')

        for f in files:
            if not f:
                continue
            f = urllib.url2pathname(f)
            f = f.strip('\r')
            f = f.replace("file://", "")
            f = f.replace("'", r"'\''")
            f = "'" + f + "'"
            filenames.append(f)

        os.system("mintupload \"" + self.service['name'] + "\" " + " ".join(filenames) + " &")

    def destroy_cb(self, wid):
        del self.drop_zones[self.service['name']]

MainClass()
Gtk.main()



#!/usr/bin/python2

import sys
import os
import time
import traceback

try:
    import gi
    gi.require_version("Gtk", "3.0")
    from gi.repository import Gtk, Gdk
except:
    print "You do not have all the dependencies!"
    sys.exit(1)

from mintupload_core import *

Gdk.threads_init()
__version__ = VERSION
# i18n
gettext.install("mintupload", "/usr/share/linuxmint/locale")

# Location of the UI file
UI_FILE = "/usr/share/linuxmint/mintupload/mintupload.ui"


def notify(message, timeout=3000):
    os.system("notify-send \"" + _("Upload Manager") + "\" \"" + message + "\" -t " + str(timeout))


class GtkUploader(MintUploader):

    '''Wrapper for the gtk management of MintUploader'''

    def __init__(self, service, files):
        MintUploader.__init__(self, service, files)
        Gdk.threads_enter()
        try:
            self.builder = Gtk.Builder()
            self.builder.add_from_file(UI_FILE)

            if len(filenames) > 1:
                title = _("%(percentage)s of %(number)d files - Uploading to %(service)s") % {'percentage': '0%', 'number': len(filenames), 'service': "\"" + service['name'] + "\""}
            else:
                title = _("%(percentage)s of 1 file - Uploading to %(service)s") % {'percentage': '0%', 'service': "\"" + service['name'] + "\""}

            self.builder.get_object("main_window").set_title(title)
            self.builder.get_object("main_window").set_icon_name(ICON)

            self.progressbar = self.builder.get_object("progressbar")

            self.builder.get_object("main_window").connect("destroy", self.close_window)
            self.builder.get_object("main_window").connect("delete_event", self.close_window)
            self.builder.get_object("cancel_button").connect("clicked", self.cancel)
        finally:
            Gdk.threads_leave()

    def run(self):

        Gdk.threads_enter()
        try:
            self.progressbar.show()
        finally:
            Gdk.threads_leave()

        # Calculate total size
        self.num_files_left = len(self.files)
        self.total_size = 0
        self.size_so_far = 0
        self.size_of_finished_files_so_far = 0
        self.percentage = 0
        self.cancel_required = False
        self.start_time = time.time()

        try:
            for f in self.files:
                self.total_size += os.path.getsize(f)

            for f in self.files:
                if self.cancel_required:
                    notify(_("The upload to '%(service)s' was cancelled") % {'service': service['name']})
                    Gtk.main_quit()
                    sys.exit(0)
                else:
                    # Upload 1 file
                    filename = os.path.split(f)[1]
                    self.upload(f)

                self.num_files_left -= 1
                self.size_of_finished_files_so_far += os.path.getsize(f)

        except Exception, e:
            notify((_("Upload to '%s' failed: ") % service['name']) + str(e))
            traceback.print_exc()
            Gtk.main_quit()
            sys.exit(0)

        if len(self.files) > 1:
            notify(_("Successfully uploaded %(number)d files to '%(service)s'") % {'number': len(self.files), 'service': service['name']})
        else:
            notify(_("Successfully uploaded 1 file to '%(service)s'") % {'service': service['name']})
            
        Gtk.main_quit()
        sys.exit(0)

    def close_window(self, widget=None, event=None):
        if self.cancel_required:
            self.builder.get_object("main_window").hide()
        else:
            self.builder_cancel = self.builder.get_object("close_dialog")
            self.builder_cancel.get_object("close_dialog").set_icon_name(ICON)
            self.builder_cancel.get_object("label_cancel").set_text(_("Do you want to cancel this upload?"))
            self.builder_cancel.get_object("cancel_button").set_label(_("Cancel"))
            self.builder_cancel.get_object("continue_button").set_label(_("Run in the background"))
            self.builder_cancel.get_object("cancel_button").connect("clicked", self.hide_window, True)
            self.builder_cancel.get_object("continue_button").connect("clicked", self.hide_window, False)
            self.builder_cancel.get_object("close_dialog").set_title(_("Cancel upload?"))
            self.builder_cancel.get_object("close_dialog").show()
        return True

    def hide_window(self, widget, cancel):
        self.builder_cancel.get_object("close_dialog").hide()
        if cancel:
            self.cancel(widget)
        self.builder.get_object("main_window").hide()

    def cancel(self, widget):
        self.cancel_required = True
        self.builder.get_object("cancel_button").set_sensitive(False)

    def progress(self, message, color=None):
        pass

    def pct(self, so_far, total=None):
        percentage = str(int(self.percentage * 100)) + "%"
        if self.num_files_left > 1:
            message = _("Uploading %(number)d files to %(service)s") % {'number': self.num_files_left, 'service': "\"" + service['name'] + "\""}
            title = _("%(percentage)s of %(number)d files - Uploading to %(service)s") % {'percentage': percentage, 'number': self.num_files_left, 'service': "\"" + service['name'] + "\""}
        else:
            message = _("Uploading 1 file to %(service)s") % {'service': "\"" + service['name'] + "\""}
            title = _("%(percentage)s of 1 file - Uploading to %(service)s") % {'percentage': percentage, 'service': "\"" + service['name'] + "\""}

        self.percentage = float(self.size_so_far) / float(self.total_size)
        Gdk.threads_enter()
        try:
            self.progressbar.set_fraction(self.percentage)
            self.progressbar.set_text(str(int(self.percentage * 100)) + "%")
            self.builder.get_object("upload_label").set_text(message)
            self.builder.get_object("main_window").set_title(title)
        finally:
            Gdk.threads_leave()
        pass

    def common_callback(self):
        self.pct(self.size_so_far, self.total_size)
        self.calculate_time()

        if self.speed > 0 and self.time_remaining > 0:
            message = _("%(size_so_far)s of %(total_size)s - %(time_remaining)s left (%(speed)s/sec)") % {'size_so_far': self.size_to_string(self.size_so_far, 1), 'total_size': self.size_to_string(self.total_size, 1), 'time_remaining': self.time_to_string(self.time_remaining), 'speed': self.size_to_string(self.speed, 1)}
        else:
            message = _("%(size_so_far)s of %(total_size)s") % {'size_so_far': self.size_to_string(self.size_so_far, 1), 'total_size': self.size_to_string(self.total_size, 1)}

        Gdk.threads_enter()
        try:
            self.builder.get_object("label_details").set_text(message)
        finally:
            Gdk.threads_leave()
        return

    def my_ftp_callback(self, buffer):
        self.size_so_far += len(buffer) - 1
        self.common_callback()
        return

    def my_sftp_callback(self, so_far, total=None):
        self.size_so_far = self.size_of_finished_files_so_far + so_far
        self.common_callback()
        return

    def success(self):
        pass

    def size_to_string(self, size, decimals):
        size = float(size)
        kilo = float(1024)
        mega = float(1024 * 1024)
        giga = float(1024 * 1024 * 1024)
        strSize = str(size) + _("B")
        if size >= kilo:
            strSize = str(round(size / kilo, decimals)) + _("KB")
        if size >= mega:
            strSize = str(round(size / mega, decimals)) + _("MB")
        if size >= giga:
            strSize = str(round(size / giga, decimals)) + _("GB")
        return strSize

    def time_to_string(self, time):
        hours, remainder = divmod(time, 3600)
        minutes, seconds = divmod(remainder, 60)
        if time > 7200:
            str = _("%(hours)d hours, %(minutes)d minutes") % {'hours': hours, 'minutes': minutes}
        elif time > 3600:
            str = _("1 hour, %d minutes") % minutes
        elif time > 120:
            str = _("%(minutes)d minutes, %(seconds)d seconds") % {'minutes': minutes, 'seconds': seconds}
        elif time > 60:
            str = _("1 minute, %d seconds") % seconds
        else:
            str = _("%d seconds") % seconds
        return str

    def calculate_time(self):
        self.size_remaining = self.total_size - self.size_so_far
        time_spent = time.time() - self.start_time
        if time_spent > 0:
            self.speed = float(self.size_so_far) / float(time_spent)
        else:
            self.speed = 0
        if self.speed > 0:
            self.time_remaining = float(self.size_remaining) / float(self.speed)
        else:
            self.time_remaining = 0

if __name__ == "__main__":
    if len(sys.argv) < 3:
        print """Usage: mintupload service file [more files]"""
        exit(0)

    service_name = sys.argv[1]
    service = None
    known_services = read_services()
    for known_service in known_services:
        if known_service['name'] == service_name:
            service = known_service

    if service is None:
        print "Unknown service: " + service_name
        os.system("notify-send \"" + _("Unknown service: %s") % service_name + "\"")
    else:
        filenames = sys.argv[2:]

        uploader = GtkUploader(service, filenames)
        uploader.start()
        Gtk.main()



# mintUpload
#       Clement Lefebvre <root@linuxmint.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; Version 3
# of the License.


import os
import sys
import urllib
import ftplib
import datetime
import gettext
import paramiko
import pexpect
import threading
import gi
gi.require_version('Notify', '0.7')
from gi.repository import Notify

from configobj import ConfigObj

USER_HOME = os.path.expanduser('~')

VERSION = "3.7.4"
__version__ = VERSION

# i18n
gettext.install("mintupload", "/usr/share/linuxmint/locale")

ICON = "mintupload"
CONFIGFILE_GLOBAL = '/etc/linuxmint/mintUpload.conf'
CONFIGFILE_USER = USER_HOME + '/.linuxmint/mintUpload.conf'


class CustomError(Exception):

    '''All custom defined errors'''

    observers = []

    def __init__(self, summary, err=None):
        self.type = self.__class__.__name__
        self.summary = summary

        self.detail = '' if not err else repr(err)

        for observer in self.observers:
            observer.error(self)

    @classmethod
    def add_observer(cls, observer):
        cls.observers.append(observer)


class CliErrorObserver:

    '''All custom defined errors, using stderr'''

    def error(self, err):
        sys.stderr.write(os.linesep + err.type + ': ' + err.summary)

        if err.detail:
            sys.stderr.write(os.linesep + '\tDetail: ' + err.detail)

        sys.stderr.write(os.linesep * 2)

CustomError.add_observer(CliErrorObserver())


class ConnectionError(CustomError):

    '''Raised when an error has occured with an external connection'''
    pass


class FilesizeError(CustomError):

    '''Raised when the file is too large or too small'''
    pass


def get_size_str(size, acc=None, factor=None):
    '''Converts integer filesize in bytes to textual repr'''

    if not factor:
        factor = int(config['filesize']['factor'])
    if not acc:
        acc = int(config['filesize']['accuracy'])
    if config['filesize']['binary_units'] == "True":
        thresholds = [_("B"), _("KiB"), _("MiB"), _("GiB")]
    else:
        thresholds = [_("B"), _("KB"), _("MB"), _("GB")]
    size = float(size)
    for i in reversed(range(1, len(thresholds))):
        if size >= factor**i:
            rounded = round(size / factor**i, acc)
            return str(rounded) + thresholds[i]
    return str(int(size)) + thresholds[0]


class MintNotifier:

    '''Enables integration with external notifiers'''

    def __init__(self):
        Notify.init("mintUpload")

    def notify(self, detail):
        Notify.Notification("mintUpload", detail, ICON).show()


class MintSpaceChecker(threading.Thread):

    '''Checks that the filesize is ok'''

    def __init__(self, service, filesize):
        threading.Thread.__init__(self)
        self.service = service
        self.filesize = filesize

    def run(self):
        try:
            self.check()
            return True
        except FilesizeError:
            return False

    def check(self):
        # Get the maximum allowed self.filesize on the service
        if self.service.has_key("maxsize"):
            if self.filesize > self.service["maxsize"]:
                raise FilesizeError(_("File larger than service's maximum"))

        # Get the available space left on the service
        if self.service.has_key("space"):
            try:
                spaceInfo = urllib.urlopen(self.service["space"]).read()
                spaceInfo = spaceInfo.split("/")
                self.available = int(spaceInfo[0])
                self.total = int(spaceInfo[1])
            except Exception as e:
                raise ConnectionError(_("Could not get available space"), e)

            if self.filesize > self.available:
                raise FilesizeError(_("File larger than service's available space"))


class MintUploader(threading.Thread):

    '''Uploads the file to the selected service'''

    def __init__(self, service, files):
        threading.Thread.__init__(self)
        service = service.for_upload()
        self.service = service
        self.focused = True
        self.files = files

        # Switch to required connect function, depending on service
        self.uploader = {
            'MINT': self._ftp,  # For backwards compatiblity
            'FTP': self._ftp,
            'SFTP': self._sftp,
            'SCP': self._scp}[self.service['type']]

    def run(self):
        for f in self.files:
            self.upload(f)

        self.progress(_("File uploaded successfully."))

    def upload(self, file):
        self.name = os.path.basename(file)
        self.filesize = os.path.getsize(file)
        self.uploader(file)
        self.success()

    def _ftp(self, file):
        '''Connection process for FTP services'''

        if not self.service.has_key('port'):
            self.service['port'] = 21
        try:
            # Attempting to connect
            ftp = ftplib.FTP()
            ftp.connect(self.service['host'], self.service['port'])
            ftp.login(self.service['user'], self.service['pass'])
            self.progress(self.service['type'] + " " + _("connection successfully established"))

            # Create full path
            for dir in self.service['path'].split(os.sep):
                try:
                    ftp.mkd(dir)
                except:
                    pass
                ftp.cwd(dir)

            f = open(file, "rb")
            self.progress(_("Uploading the file..."))
            self.pct(0)
            self.so_far = 0
            ftp.storbinary('STOR ' + self.name, f, 1024, callback=self.my_ftp_callback)

        finally:
            # Close any open connections
            try:
                f.close()
            except:
                pass

            try:
                ftp.quit()
            except:
                pass

    def _sftp(self, file):
        '''Connection process for SFTP services'''
        if not self.service.has_key('port'):
            self.service['port'] = 22
        try:
            ssh = paramiko.SSHClient()
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            key_filename = os.path.expanduser("~/.ssh/id_rsa")
            if os.path.exists(key_filename):
                ssh.connect(self.service['host'], username=self.service['user'], password=self.service['pass'], port=self.service['port'], key_filename=key_filename)
            else:
                ssh.connect(self.service['host'], username=self.service['user'], password=self.service['pass'], port=self.service['port'])      

            sftp = ssh.open_sftp()
            self.progress(_("Uploading the file..."))
            self.pct(0)
            sftp.put(file, self.service['path'] + self.name, self.my_sftp_callback)

        finally:
            # Close any open connections
            try:
                sftp.close()
            except:
                pass

            try:
                ssh.close()
            except:
                pass

    def _scp(self, file):
        '''Connection process for SCP services'''

        if not self.service.has_key('port'):
            self.service['port'] = 22

        try:
            # Attempting to connect
            self.service['file'] = file
            scp = pexpect.spawn("scp", ["-P", "%(port)i" % self.service,
                                        "%(file)s" % self.service,
                                        "%(user)s@%(host)s:%(path)s" % self.service])

            # If password is not defined, or is the empty string, use password-less scp
            if self.service['pass']:
                scp.expect('.*password:*')
                scp.sendline(self.service['pass'])

            self.progress(self.service['type'] + " " + _("connection successfully established"))

            scp.timeout = None
            self.pct(0)
            received = scp.expect(['.*100\%.*', '.*password:.*', pexpect.EOF])

            if received == 1:
                scp.sendline(' ')
                raise ConnectionError(_("This service requires a password."))

        finally:
            # Close any open connections
            try:
                scp.close()
            except:
                pass

    def progress(self, message):
        print message

    def pct(self, so_far, total=None):
        if not total:
            total = self.filesize

        if total:
            pct = float(so_far) / total
        else:
            pct = 1.0

        pct = int(pct * 100)
        sys.stdout.write("\r " + str(pct) + "% [" + (pct / 2) * "=" + ">" + (50 - (pct / 2)) * " " + "] " + get_size_str(so_far) + "     ")
        sys.stdout.flush()
        return pct

    def success(self):
        self.pct(self.filesize)
        sys.stdout.write("\n")
        # Print URL
        if self.service.has_key('url'):
            url = self.service['url'].replace('<FILE>', self.name)
            self.url = url.replace(' ', '%20')
            self.progress(_("URL:") + " " + self.url)

        n = config['notification']
        # If nofications are enabled AND the file is minimal x byte in size...
        if n['enable'] == "True" and self.filesize >= int(n['min_filesize']):
            # If when_focused is true OR window has no focus
            if n['when_focused'] == "True" or not self.focused:
                MintNotifier().notify(_("File uploaded successfully."))

    def my_ftp_callback(self, buffer):
        self.so_far = self.so_far + len(buffer) - 1
        self.pct(self.so_far)
        return

    def my_sftp_callback(self, so_far, total=None):
        return self.pct(so_far, total)


def read_services():
    '''Get all defined services'''

    services = []
    for loc, path in config_paths.iteritems():
        os.system("mkdir -p " + path)
        for file in os.listdir(path):
            try:
                s = Service(path + file)
            except:
                pass
            else:
                s['loc'] = loc
                services.append(s)
    return services


config = ConfigObj(CONFIGFILE_GLOBAL)
if os.path.exists(CONFIGFILE_USER):
    config.merge(ConfigObj(CONFIGFILE_USER))

if not config.has_key('paths'):
    print _("%(1)s is not set in the config file found under %(2)s or %(3)s") % {'1': 'paths', '2': CONFIGFILE_GLOBAL, '3': CONFIGFILE_USER}
    sys.exit(1)

if not config.has_key('defaults'):
    print _("%(1)s is not set in the config file found under %(2)s or %(3)s") % {'1': 'defaults', '2': CONFIGFILE_GLOBAL, '3': CONFIGFILE_USER}
    sys.exit(1)

config_paths = config['paths']
config_paths['user'] = config_paths['user'].replace('<HOME>', USER_HOME)

defaults = config['defaults']
defaults['user'] = defaults['user'].replace('<USER>', os.environ['USER'])


class Service(ConfigObj):

    '''Object representing an upload service'''

    def __init__(self, *args):
        ConfigObj.__init__(self, *args)
        self._fix()

    def merge(self, *args):
        ConfigObj.merge(self, *args)
        self._fix()

    def remove(self):
        os.system("rm '" + self.filename + "'")

    def move(self, newname, force=False):
        if force or not os.path.exists(newname):
            os.system("mv '" + self.filename + "' '" + newname + "'")
            self.filename = newname

    def copy(self, newname, force=False):
        if force or not os.path.exists(newname):
            oldname = self.filename
            self.filename = newname
            self.write()
            self.filename = oldname

    def _fix(self):
        '''Format values correctly'''

        for k, v in self.iteritems():
            if v:
                if type(v) is list:
                    self[k] = ','.join(v)
            else:
                self.pop(k)

        if self.filename:
            self['name'] = os.path.basename(self.filename)

        if self.has_key('type'):
            self['type'] = self['type'].upper()

        if self.has_key('host'):
            h = self['host']
            if h.find(':') >= 0:
                h = h.split(':')
                self['host'] = h[0]
                self['port'] = h[1]

        ints = ['port', 'maxsize', 'persistence']
        for k in ints:
            if self.has_key(k):
                self[k] = int(self[k])

    def for_upload(self):
        '''Prepare a service for uploading'''

        s = defaults
        s.merge(self)

        timestamp = datetime.datetime.utcnow().strftime(s['format'])
        s['path'] = s['path'].replace('<TIMESTAMP>', timestamp)

        # Replace placeholders in url
        if s.has_key('url'):
            url_replace = {
                '<TIMESTAMP>': timestamp,
                '<PATH>': s['path']
            }
            url = s['url']
            for k, v in url_replace.iteritems():
                url = url.replace(k, v)
            # Must be done after other replaces to function correctly
            url = url.replace(' ', '%20')
            s['url'] = url

        # Ensure trailing '/', after url <PATH> replace
        if s['path']:
            s['path'] = os.path.normpath(s['path'])
        else:
            s['path'] = os.curdir
        s['path'] += os.sep

        return s


def _my_storbinary(self, cmd, fp, blocksize=8192, callback=None):
    '''Store a file in binary mode.'''

    self.voidcmd('TYPE I')
    conn = self.transfercmd(cmd)
    while True:
        buf = fp.read(blocksize)

        if not buf:
            break

        conn.sendall(buf)

        if callback:
            callback(buf)

    conn.close()
    return self.voidresp()


def _my_storlines(self, cmd, fp, callback=None):
    '''Store a file in line mode.'''

    self.voidcmd('TYPE A')
    conn = self.transfercmd(cmd)
    while 1:
        buf = fp.readline()
        if not buf:
            break
        if buf[-2:] != CRLF:     # CRLF is defined in ftplib.  This code is valid
            if buf[-1] in CRLF:  # after being patched into that context.  See below
                buf = buf[:-1]
            buf = buf + CRLF
        conn.sendall(buf)
        if callback:
            callback(buf)
    conn.close()
    return self.voidresp()

# Use the patched versions
ftplib.FTP.storbinary = _my_storbinary
ftplib.FTP.storlines = _my_storlines
